package dev.fyloz.colorrecipesexplorer.utils

import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule
import com.fasterxml.jackson.module.kotlin.jacksonMapperBuilder
import com.fasterxml.jackson.module.kotlin.readValue
import dev.fyloz.colorrecipesexplorer.model.account.User
import dev.fyloz.colorrecipesexplorer.model.account.UserOutputDto
import dev.fyloz.colorrecipesexplorer.model.account.toOutputDto
import io.jsonwebtoken.Jwts
import io.jsonwebtoken.io.Encoders
import io.jsonwebtoken.jackson.io.JacksonDeserializer
import io.jsonwebtoken.jackson.io.JacksonSerializer
import io.jsonwebtoken.security.Keys
import java.util.*
import javax.crypto.SecretKey

private const val userClaimName = "user"

private val objectMapper = jacksonMapperBuilder()
    .apply { addModule(JavaTimeModule()) }
    .build()
private val serializer = JacksonSerializer<Map<String, *>>(objectMapper)
private val deserializer = JacksonDeserializer<Map<String, *>>(objectMapper)

/** Build a JWT token for the given [User]. */
fun User.buildJwt(secret: String, duration: Long): String {
    val expirationDate = Date(System.currentTimeMillis() + duration)
    val serializedUser = objectMapper.writeValueAsString(this.toOutputDto())

    return Jwts.builder()
        .serializeToJsonWith(serializer)
        .signWith(keyFromSecret(secret))
        .setSubject(this.id.toString())
        .setExpiration(expirationDate)
        .claim(userClaimName, serializedUser)
        .compact()
}

/** Parses the user of the given [jwt]. */
fun parseJwtUser(jwt: String, secret: String): UserOutputDto =
    with(
        Jwts.parserBuilder()
            .deserializeJsonWith(deserializer)
            .setSigningKey(keyFromSecret(secret))
            .build()
            .parseClaimsJws(jwt)
            .body.get(userClaimName, String::class.java)
    ) {
        objectMapper.readValue(this)
    }

/** Creates a base64 encoded [SecretKey] from the given [secret]. */
private fun keyFromSecret(secret: String): SecretKey =
    with(Encoders.BASE64.encode(secret.toByteArray())) {
        Keys.hmacShaKeyFor(this.toByteArray())
    }
